!     Last change:  DC   17 Dec 2002    8:58 am
       module slabpar
        implicit none
        INTEGER :: nlayers, callrad=0
        REAL, DIMENSION(10) :: h, hcap, cond
        COMPLEX, parameter :: i = (0.0, 1.0)
        REAL, parameter :: pi=3.143159
        real :: w0, htx
        COMPLEX, DIMENSION(10) :: k2
       end module slabpar

       program multi3w

!            calculates  thermal response of mulilayer slab in a 3w-type measurement

       USE slabpar
       implicit none

       interface
        subroutine gausscalc(omega,tt)
         implicit none
         COMPLEX , INTENT(out) :: tt
         REAL, INTENT(IN) :: omega
        end subroutine gausscalc
       END interface

       integer :: j

       complex :: temp
       real ::  pumpf, omega
       REAL, parameter :: pi2=2*3.14159, stepsiz = 1.2589254117941672

        open(1, status='old', file='multi3w.par')

!
!           first get the half linewidth and the heater/thermometer separation

        read(1,*) w0, htx

        write(*,*) 'half linewidth is', w0*1e4, ' microns'
        WRITE(*,*) 'heater thermometer separation is', htx*1e4, ' microns'
!
!                 then read in the properties of each layer
!
        write(*,*) 'conductivity, heat capacity, thickness:'
        do j = 1,10
         read(1,*,end=999) cond(j),hcap(j),h(j)
         write(*,110) cond(j), hcap(j), h(j)
 110     FORMAT(3(ES12.3))
        end do
!
!                 define values of the nlayer+1 layer for zero heat flow at bottom
!
 999    cond(j) = 0.0
        hcap(j) = 1.0
        h(j) = 0.0
        nlayers = j-1
        write(*,*) 'total number of layers:', nlayers
        write(*,*) ' '


!               open a file for the data

       open(2, status='unknown', file='multi3w.dat')

       pumpf = 1.0
       WRITE(*,*) '              Hz, heater frequency'

       do WHILE (pumpf .LT. 1e5*stepsiz)

       WRITE(*,111) pumpf
 111   FORMAT('+', E12.2 )

!                 loop over all frequencies starting with 1 Hz

        omega = pumpf*pi2
        call gausscalc(omega,temp)

        temp = temp/w0**2/3.14159

        write(2,100) pumpf, temp
 100    FORMAT(3(ES15.4))

        pumpf=pumpf*stepsiz

        END do
        WRITE(*,*) ' made', callrad, ' calls to function radial'

       stop
       end program multi3w

!---------------------------------------------------------------------------------
       subroutine gausscalc(omega,tt)

       USE slabpar
       USE nr, ONLY : qromo, midpnt, midinf
!
!              subroutine to calculate the complex temperature of a multilayer
!              slab at frequency omega, full calculation or radial heat flow
!
       implicit none
       interface
         function radialr(m)
         USE nrtype
         implicit none
         REAL(SP), DIMENSION(:), INTENT(IN) :: m
         REAL(SP), DIMENSION(size(m)) :: radialr
        END function radialr
        end interface

        interface
         function radiali(m)
         USE nrtype
         implicit none
         REAL(SP), DIMENSION(:), INTENT(IN) :: m
         REAL(SP), DIMENSION(size(m)) :: radiali
        END function radiali
        end interface

        interface
          function radial(m1)
          implicit none
          REAL, INTENT(IN) :: m1
          COMPLEX :: radial
          END function radial
        end interface


       REAL, INTENT(IN) :: omega
       COMPLEX, INTENT(out) :: tt
       INTEGER :: j
       REAL :: ttr1, ttr2, tti1, tti2, cross

!                calculate iw/D for each layer at this frequency w

       do j=1, nlayers
        k2(j) = i*omega/cond(j)*hcap(j)
       end do

!                do the real and imaginary parts of the integration

        cross = 1.0/w0
        ttr1 = qromo( radialr, 0.0, cross, midpnt )
        ttr2 = qromo( radialr, cross, 1e6/w0, midinf )
        tti1 = qromo( radiali, 0.0, cross, midpnt )
        tti2 = qromo( radiali, cross, 1e6/w0, midinf )

        tt = ttr1+ttr2+i*(tti1+tti2)

       return
       END subroutine gausscalc
!
!--------------------------------------------------------------------------
!
       function radial(m)
        USE slabpar
        implicit none

        interface
         function u(j,m2)
         implicit none
         COMPLEX :: u
         integer, INTENT(IN) :: j
         REAL, INTENT(IN) :: m2
        END function u                                               
        end interface

        real, INTENT(IN) :: m
        COMPLEX :: radial
        REAL :: m2

        COMPLEX, DIMENSION(2) :: B, Btemp, Btemp2
        COMPLEX, DIMENSION(2,2) :: alpha, beta
        INTEGER :: j
        COMPLEX :: gj, gjp1, uj, ujp1

!             keep track of how many times this function is called

        callrad=callrad+1

!              initialize the transfer matrix and the B vector

       B(1) = CMPLX(0.0, 0.0)
       B(2) = CMPLX(1.0, 0.0)
       beta(1,2) = CMPLX(0.0, 0.0)
       beta(2,1) = CMPLX(0.0, 0.0)

       m2 = m**2

       j = nlayers

       do WHILE(j .ge. 1)

        uj = u(j, m2)

        IF( REAL(uj)*h(j) .LT. 7.0 ) then

!             calculate the new B vector iteratively

           ujp1 = u(j+1, m2)

           gjp1 = cond(j+1)*ujp1
           gj = cond(j)*uj

           alpha(1,1) = gj+gjp1
           alpha(1,2) = gj-gjp1
           alpha(2,1) = alpha(1,2)
           alpha(2,2) = alpha(1,1)

           Btemp = matmul(alpha,B)

           beta(1,1) = EXP(-uj*h(j))
           beta(2,2) = 1.0/beta(1,1)

           Btemp2 = MATMUL(beta,Btemp)

           B = Btemp2/gj
        else

!              heat flow doesn't make it to the bottom of this layer so reset B vector

           B(1) = CMPLX(0.0, 0.0)
           B(2) = CMPLX(1.0, 0.0)
        END if

        j = j-1

        END do

        radial = (B(1)+B(2))/(B(2)-B(1))/u(1,m2)/cond(1)*COS(m*htx)

       return
       END function radial

!-------------------------------------------------------------------------------

       function u(j,m2)
       USE slabpar
       implicit none
       INTEGER, INTENT(IN) :: j
       REAL, INTENT(IN) :: m2
       COMPLEX :: u

       u = (k2(j)+m2)**0.5
       return
       end function u

!----------------------------------------------------------------------------------

       function radiali(m)
         USE slabpar
         use nrtype
         implicit none

         interface
            function radial(m1)
            implicit none
            REAL, INTENT(IN) :: m1
            COMPLEX :: radial
            END function radial
         end interface

         REAL(SP), DIMENSION(:), INTENT(IN) :: m
         REAL(SP), DIMENSION(size(m)) :: radiali
         REAL :: msc
         INTEGER :: j

         do j = 1, SIZE(m)
           msc = m(j)
           radiali(j) = aimag(radial(msc))*SIN(msc*w0)**2/msc**2
         end do

         return
        END function radiali
!
!---------------------------------------------------------------------------------
!
       function radialr(m)
         USE slabpar
         use nrtype
         implicit none

         interface
            function radial(m1)
            implicit none
            REAL, INTENT(IN) :: m1
            COMPLEX :: radial
            END function radial
         end interface

         REAL(SP), DIMENSION(:), INTENT(IN) :: m
         REAL(SP), DIMENSION(size(m)) :: radialr
         INTEGER :: j
         REAL :: msc

         do j = 1, SIZE(m)
           msc = m(j)
           radialr(j) = real(radial(msc))*SIN(msc*w0)**2/msc**2
         end do

         return
        END function radialr
